@@ -1,6 +1,6 @@
 # ClamTk, copyright (C) 2004-2011 Dave M
 #
-# This file is part of ClamTk.
+# This file is part of ClamTk (http://clamtk.sourceforge.net).
 #
 # ClamTk is free software; you can redistribute it and/or modify it
 # under the terms of either:
@@ -18,10 +18,15 @@ $| = 1;
 use encoding 'utf8';
 
 use Net::DNS;
+use Date::Calc 'Date_to_Days';
+use File::Basename 'basename';
+
+use POSIX 'locale_h';
+use Locale::gettext;
 
 sub get_TK_version {
     # Stick with %.2f format - 4.50 vice 4.5
-    return '4.32';
+    return '4.35';
 }
 
 sub get_path {
@@ -93,23 +98,30 @@ sub def_paths {
     # As of 4.23, we're no longer looking for the .info dirs.
     # The .cvd is the compressed database, while .cld is a
     # previous .cvd/.cld with incremental updates.
+    # The problem is that you can end up with both a
+    # daily.cvd AND a daily.cld, and it's a crapshoot as to
+    # which one will show the most current date.  So we'll return
+    # just the directory path, compare dates, and return
+    # the most current date.
 
     my ( $DAILY_PATH, $MAIN_PATH );
 
     # These are the typical directories where the sigs are found.
-	# Because CentOS is a little screwy, it will often contain two
-	# directories of definitions... The newer one is likely in
-	# /var/clamav, so check that first.  Other distros will skip it
-	# and will likely find the defs under /var/lib/clamav.
-    my @dirs = qw#
+    # Because CentOS is a little screwy, it will often contain two
+    # directories of definitions... The newer one is likely in
+    # /var/clamav, so check that first.  Other distros will
+    # likely find the defs under /var/lib/clamav.
+    my @dirs = qw(
         /var/clamav
         /var/lib/clamav
         /opt/local/share/clamav
         /usr/share/clamav
         /usr/local/share/clamav
         /var/db/clamav
-        #;
+        );
 
+    # If the user selected "manual", that directory needs
+    # to be checked first, so we'll jam that in with unshift.
     my $user_set = 0;
     if ( ClamTk::Prefs->get_preference('Update') eq 'single' ) {
         $user_set = 1;
@@ -121,9 +133,9 @@ sub def_paths {
     # and check for .cld before .cvd
     for my $dir_list (@dirs) {
         if ( -e "$dir_list/daily.cld" ) {
-            $DAILY_PATH = "$dir_list/daily.cld";
+            $DAILY_PATH = $dir_list;
         } elsif ( -e "$dir_list/daily.cvd" ) {
-            $DAILY_PATH = "$dir_list/daily.cvd";
+            $DAILY_PATH = $dir_list;
         }
         if ( -e "$dir_list/main.cld" ) {
             $MAIN_PATH = "$dir_list/main.cld";
@@ -147,11 +159,18 @@ sub get_AV_version {
     # past without me noticing...
     local $ENV{'PATH'} = '/bin:/usr/bin:/usr/local/bin';
     delete @ENV{ 'IFS', 'CDPATH', 'ENV', 'BASH_ENV' };
-    my $paths = ClamTk::App->get_path('clampath');
-    my ($version) = qx | $paths -V |;
-    $version =~ s/ClamAV\s(.*?)\/.*/$1/;
-    chomp($version);
-    return $version;
+    my $paths   = ClamTk::App->get_path('clampath');
+    my $version = '';
+
+    if ( open( my $c, '-|', $paths, '-V' ) ) {
+        while (<$c>) {
+            chomp;
+            $version = $_;
+        }
+    }
+
+    $version =~ s/^\S+\s+([0-9\.]+).*/$1/;
+    return $version ? $version : '0.00';
 }
 
 sub get_AV_remote {
@@ -168,8 +187,8 @@ sub get_AV_remote {
 
     # No longer need the alarm to "die" since Net::DNS
     # does it for us.
-    # Just to be sure, we'll use OpenDNS servers here
-    # and set a UDP timeout.
+    # Just to be sure, we'll set a UDP timeout.
+    # 3 seconds ought to be enough for anyone.
     my ( $get, $res );
 
     $res = Net::DNS::Resolver->new( udp_timeout => 3, );
@@ -178,7 +197,7 @@ sub get_AV_remote {
         $res->nameservers(
             qw(208.67.222.222 208.67.220.220
                 208.67.222.220 208.67.220.222)
-        );
+                );
     }
 
     ($get) = $res->send( $rmt_host, "TXT" );
@@ -191,7 +210,7 @@ sub get_AV_remote {
   # This is a sample of the line that is returned ($get->string):
   # current.cvd.clamav.net.	439	IN	TXT	"0.95.3:51:10091:1259420373:1:44:12728"
   # We just need the version, held between the " and the :
-    if ( $get->string =~ /IN\s+TXT\s+\"(.*?):/ ) {
+    if ( $get->string =~ /IN\s+TXT\s+\"([\d\.]+):/ ) {
         return $1;
     } else {
         return 'undef';
@@ -215,13 +234,15 @@ sub get_sig_info {
     # This sub parses the signature files for the date
     # and number of signatures.
     my ( $daily_path, $main_path ) = def_paths();
-    #warn "I'm missing daily.{cld,cvd} and/or main.{cld,cvd}.\n"
+
+    # warn "using >$daily_path< for definitions.\n";
+    # warn "I'm missing daily.{cld,cvd} and/or main.{cld,cvd}.\n"
     #    unless ( $daily_path && $main_path );
 
     # $INFO_DATE = Date of signatures
     # $INFO_DAILY = # of 'daily' signatures (not part of main.{cld,cvd})
     # $INFO_MAIN = # of signatures part of 'main' signatures
-    my ( $INFO_DATE, $INFO_DAILY, $INFO_MAIN );
+    my ( $INFO_DATE, $INFO_DAILY, $INFO_MAIN ) = (0) x 3;
 
     # example of what we need to parse
     # ClamAV-VDB:07 Nov 2009 03-23 -0500:9999:102368:44:X:X:ccordes:1257582210
@@ -249,25 +270,106 @@ sub get_sig_info {
         $INFO_MAIN = 0;
     }
 
-    if ($daily_path) {
-        my $FILE;
-        if ( open( $FILE, '<', $daily_path ) ) {
-            binmode($FILE);
-            while (<$FILE>) {
-                if (/ClamAV-VDB:(\S+\s+\S+\s+\S+).*[\-\+]\d+:\d+:(\d+)/) {
-                    $INFO_DATE  = $1;
-                    $INFO_DAILY = $2;
-                    last;
-                }
+    my ( $cld_day, $cld_mon, $cld_year );
+    my ( $cvd_day, $cvd_mon, $cvd_year );
+
+    # Check for .cld
+    my $FILE;
+    if ( open( $FILE, '<', "$daily_path/daily.cld" ) ) {
+        binmode($FILE);
+        while (<$FILE>) {
+            if (/ClamAV-VDB:(\S+\s+\S+\s+\S+).*[\-\+]\d+:\d+:(\d+)/) {
+                ( $cld_day, $cld_mon, $cld_year ) = split( /\s+/, $1 );
+                $INFO_DAILY = $2;
+                last;
             }
-            close($FILE)
-                or warn "Couldn't close $daily_path: $!\n";
-        } else {
-            $INFO_DATE  = '01 Jan 1970';
-            $INFO_DAILY = 0;
         }
+        close($FILE)
+            or warn "Couldn't close $daily_path: $!\n";
     }
+    if ( !$cld_day ) {
+        $cld_day  = '01';
+        $cld_mon  = 'Jan';
+        $cld_year = '1900';
+    }
+    # warn "cld: $cld_day $cld_mon $cld_year\n";
+
+    # Check for .cvd
+    if ( open( $FILE, '<', "$daily_path/daily.cvd" ) ) {
+        binmode($FILE);
+        while (<$FILE>) {
+            if (/ClamAV-VDB:(\S+\s+\S+\s+\S+).*[\-\+]\d+:\d+:(\d+)/) {
+                ( $cvd_day, $cvd_mon, $cvd_year ) = split( /\s+/, $1 );
+                $INFO_DAILY = $2;
+                last;
+            }
+        }
+        close($FILE)
+            or warn "Couldn't close $daily_path: $!\n";
+    }
+    if ( !$cvd_day ) {
+        $cvd_day  = '01';
+        $cvd_mon  = 'Jan';
+        $cvd_year = '1900';
+    }
+    # warn "cvd: $cvd_day $cvd_mon $cvd_year\n";
+
+    # Now compare the dates and return the more recent one.
+    my %months = (
+        'Jan' => '01',
+        'Feb' => '02',
+        'Mar' => '03',
+        'Apr' => '04',
+        'May' => '05',
+        'Jun' => '06',
+        'Jul' => '07',
+        'Aug' => '08',
+        'Sep' => '09',
+        'Oct' => 10,
+        'Nov' => 11,
+        'Dec' => 12,
+        );
+
+    my $cmp =
+        ( Date_to_Days( $cld_year, $months{$cld_mon}, $cld_day )
+            <=> Date_to_Days( $cvd_year, $months{$cvd_mon}, $cvd_day ) );
+
+    # If cmp == -1, cvd is newer.
+    # If cmp ==  1, cld is newer.
+    # If cmp ==  0, they're the same.
+
+    $INFO_DATE =
+          ( $cmp == -1 ) ? join( ' ', $cvd_day, $months{$cvd_mon}, $cvd_year )
+        : ( $cmp == 0 )  ? join( ' ', $cvd_day, $months{$cvd_mon}, $cvd_year )
+        : ( $cmp == 1 )  ? join( ' ', $cld_day, $months{$cld_mon}, $cld_year )
+        :                  '01 01 1900';
+
     return ( $INFO_DATE, $INFO_DAILY, $INFO_MAIN );
 }
 
+sub lastscan {
+    my $path = ClamTk::App->get_path('history');
+    my @logs = glob "$path/*.log";
+    return gettext('Never') if ( !@logs );
+    my %orcs;
+
+    my @newer =
+        sort { ( $orcs{$a} ||= -M $a ) <=> ( $orcs{$b} ||= -M $b ) } @logs;
+
+    # The newest "file" (actually a string/scalar)
+    # is a path like /home/foo/.clamtk/histories/01-01-Jan.log.
+    # We just want the basename of that.
+    my $chosen = basename( $newer[0] );
+
+    my ( $month, $day, $year ) = split( /-/, $chosen );
+    $year =~ s/(\d+)\.log/$1/;
+
+    return "$day $month $year";
+}
+
+sub translate {
+    # This is a dummy routine, solely for the .desktop file.
+    return gettext('Scan for viruses...');
+}
+
 1;
