@@ -1,6 +1,6 @@
 # ClamTk, copyright (C) 2004-2011 Dave M
 #
-# This file is part of ClamTk.
+# This file is part of ClamTk (http://clamtk.sourceforge.net).
 #
 # ClamTk is free software; you can redistribute it and/or modify it
 # under the terms of either:
@@ -17,7 +17,7 @@ $| = 1;
 
 use encoding 'utf8';
 
-use File::Basename;
+use File::Basename 'basename', 'dirname';
 use File::Copy;
 use Digest::MD5 'md5_hex';
 
@@ -26,9 +26,6 @@ use Glib 'TRUE', 'FALSE';
 
 use Locale::gettext;
 use POSIX 'locale_h';
-textdomain('clamtk');
-setlocale( LC_MESSAGES, '' );
-bind_textdomain_codeset( 'clamtk', 'UTF-8' );
 
 my $popup;
 my $slist;
@@ -39,20 +36,24 @@ sub display {
     my $cmd_scan = shift;
     my ($hash) = @_;
 
-    $popup = Gtk2::Window->new();
+    $popup = Gtk2::Dialog->new( gettext('Scanning Results'),
+        undef, 'destroy-with-parent' );
     $popup->signal_connect( destroy => sub { $popup->destroy } );
-    $popup->set_default_size( 600, 250 );
+    $popup->set_default_size( 650, 250 );
     $popup->set_title( gettext('Scanning Results') );
 
+    my ( $x, $y ) = ClamTk::GUI->window_coords();
+    $popup->move( $x, $y );
+
     my $pbox = Gtk2::VBox->new();
-    $popup->add($pbox);
+    $popup->get_content_area()->add($pbox);
     $hover_label =
         ( $cmd_scan == 0 )
         ? Gtk2::Label->new( gettext('Possible threats have been found.') )
         : ( $cmd_scan == 1 )
         ? Gtk2::Label->new( gettext('The scan is complete.') )
         : '';
-    $pbox->pack_start( $hover_label, FALSE, FALSE, 5 );
+    $pbox->pack_start( $hover_label, FALSE, FALSE, 0 );
 
     # This scrolled window holds the slist
     my $scrolled_win = Gtk2::ScrolledWindow->new;
@@ -64,7 +65,7 @@ sub display {
         gettext('File')         => 'markup',
         gettext('Status')       => 'markup',
         gettext('Action Taken') => 'markup',
-    );
+        );
     $scrolled_win->add($slist);
     $scrolled_win->grab_focus();
 
@@ -74,6 +75,8 @@ sub display {
 
     $slist->set_reorderable(TRUE);
     map { $_->set_resizable(TRUE) } $slist->get_columns;
+    map { $_->set_sizing('fixed') } $slist->get_columns;
+    map { $_->set_expand(TRUE) } $slist->get_columns;
 
     # Sorting the columns.
     # Can't do it yet because it dorks up the sexy highlighting
@@ -85,7 +88,7 @@ sub display {
     $slist->set(
         hover_selection => TRUE,
         hover_expand    => TRUE
-    );
+        );
 
     # Display the possible infections
     for my $keys ( sort keys %$hash ) {
@@ -100,7 +103,6 @@ sub display {
     my $tooltips = Gtk2::Tooltips->new;
     $tooltips->set_tip( $slist,
         gettext('Select a file and right-click for options...') );
-    $tooltips->disable;
 
     # below: the right-click functionality. also uses 'sub confirm'.
     $slist->signal_connect(
@@ -133,8 +135,7 @@ sub display {
                         next if ( -d $hash->{$t}->{name} );
                         main_confirm( $t, 'q' );
                     }
-                }
-            );
+                } );
             $quar_pop->show();
             $menu->append($quar_pop)
                 unless dirname( $hash->{$deref}->{name} )
@@ -153,8 +154,7 @@ sub display {
                         next if ( -d $hash->{$u}->{name} );
                         main_confirm( $u, 'd' );
                     }
-                }
-            );
+                } );
             $delete_pop->show();
             $menu->append($delete_pop)
                 unless dirname( $hash->{$deref}->{name} )
@@ -172,7 +172,7 @@ sub display {
                         gettext('Save As...'), undef, 'save',
                         'gtk-cancel' => 'cancel',
                         'gtk-ok'     => 'ok',
-                    );
+                        );
                     $save_dialog->set_do_overwrite_confirmation(TRUE);
                     $save_dialog->set_default_response('ok');
 
@@ -193,8 +193,7 @@ sub display {
                     } else {
                         $save_dialog->destroy();
                     }
-                }
-            );
+                } );
             $save_pop->show();
             $menu->append($save_pop);
 
@@ -208,9 +207,27 @@ sub display {
                 $event->time );
 
             return TRUE;
-        }
-    );
+        } );
+
+    my $bottombar = Gtk2::Toolbar->new;
+    $bottombar->set_style('both-horiz');
+    $pbox->pack_start( $bottombar, FALSE, FALSE, 0 );
+
+    my $b_sep = Gtk2::SeparatorToolItem->new;
+    $b_sep->set_draw(FALSE);
+    $b_sep->set_expand(TRUE);
+    $bottombar->insert( $b_sep, -1 );
+
+    my $results_close_btn = Gtk2::ToolButton->new_from_stock('gtk-close');
+    $results_close_btn->set_is_important(TRUE);
+    $bottombar->insert( $results_close_btn, -1 );
+    $results_close_btn->signal_connect(
+        clicked => sub {
+            $popup->destroy;
+        } );
+
     $popup->show_all();
+    $slist->columns_autosize;
 }
 
 sub main_confirm {
@@ -329,17 +346,18 @@ sub move_to_quarantine {
         unlink("$paths/$basename");
         return;
         };
-    rename( "$paths/$basename", "$paths/$basename.VIRUS" );
 
     # Update restore file by adding file, path and md5
-    my $ctx = Digest::MD5->new;
-    open( my $F, '<', "$paths/$basename.VIRUS" ) or do {
-        warn "Could not open file for md5-ing: $!\n";
-        return;
+    my $ctx = do {
+        local $/ = undef;
+        open( my $F, '<', "$paths/$basename" ) or do {
+            warn "Could not open file for md5-ing: $!\n";
+            return;
+        };
+        binmode($F);
+        <$F>;
     };
-    $ctx->addfile(*$F);
-    my $md5 = $ctx->hexdigest;
-    close($F);
+    my $md5 = md5_hex($ctx);
 
     ClamTk::Prefs->restore( $md5, 'add', $full_name, $perm );
 
