@@ -1,6 +1,6 @@
 # ClamTk, copyright (C) 2004-2011 Dave M
 #
-# This file is part of ClamTk.
+# This file is part of ClamTk (http://clamtk.sourceforge.net).
 #
 # ClamTk is free software; you can redistribute it and/or modify it
 # under the terms of either:
@@ -22,9 +22,6 @@ use encoding 'utf8';
 
 use POSIX 'locale_h';
 use Locale::gettext;
-textdomain("clamtk");
-setlocale( LC_MESSAGES, "" );
-bind_textdomain_codeset( "clamtk", "UTF-8" );
 
 use Glib 'TRUE', 'FALSE';
 
@@ -36,8 +33,7 @@ my $cmd =
     :                                   '';
 chomp($cmd);
 
-my $window;
-my $tt;    # tooltips
+my $dialog;
 my ( $hour_s, $min_s, $hour_s_2, $min_s_2 );
 my ( $root_b, $home_b );
 my ( $scan_apply_btn, $scan_remove_btn );
@@ -46,51 +42,58 @@ my ( $scan_status,    $defs_status );
 my $ignore_white;    # checked = scan whitelisted, unchecked = use
 
 sub schedule_dialog {
-    $window = Gtk2::Window->new;
-    $window->signal_connect( destroy => sub { Gtk2->main_quit; } );
-    $window->set_title( gettext('Schedule') );
-    $window->set_position('center');
+    # These are the main $window's x/y coordinates.
+    # Seems to work intermittently, and possibly
+    # less expensive than sending $window itself.
+    # The ->move below shifts it to those coordinates.
+    my ( undef, $x, $y ) = @_;
+    my $dialog =
+        Gtk2::Dialog->new( gettext('Schedule'), undef,
+        'destroy-with-parent' );
+    $dialog->signal_connect( close   => sub { $dialog->destroy } );
+    $dialog->signal_connect( destroy => sub { Gtk2->main_quit } );
+    $dialog->move( $x, $y );
 
-    $tt = Gtk2::Tooltips->new();
+    my $tt = Gtk2::Tooltips->new();
 
     my $vbox = Gtk2::VBox->new;
-    $window->add($vbox);
+    $dialog->get_content_area()->add($vbox);
 
     my $scan_f = Gtk2::Frame->new( gettext('Scan options') );
     $vbox->pack_start( $scan_f, FALSE, FALSE, 5 );
     $scan_f->set_border_width(3);
+    #$scan_f->set_shadow_type('none');
 
     my $scan_b = Gtk2::VBox->new;
     $scan_f->add($scan_b);
 
-    my $dialog1 = Gtk2::Label->new;
-    $scan_b->pack_start( $dialog1, FALSE, FALSE, 0 );
-    $dialog1->set_text( gettext('Scan my ... ') );
+    my $label1 = Gtk2::Label->new;
+    $scan_b->pack_start( $label1, FALSE, FALSE, 0 );
+    $label1->set_text( gettext('Scan my ... ') );
 
     my $dir_bb = Gtk2::HButtonBox->new;
     $scan_b->pack_start( $dir_bb, FALSE, FALSE, 0 );
     $dir_bb->set_layout('spread');
 
     $home_b = Gtk2::RadioButton->new( undef, gettext('Home (recommended)') );
+    $home_b->can_focus(FALSE);
     $dir_bb->add($home_b);
     $tt->set_tip(
         $home_b,
         gettext(
             'This option will scan your home directory. This is the recommended option.'
-        )
-    );
+            ) );
     $root_b = Gtk2::RadioButton->new( $home_b, gettext('entire computer') );
     $dir_bb->add($root_b);
     $tt->set_tip(
         $root_b,
         gettext(
             'This option will scan your entire computer but will exclude the /proc, /sys and /dev directories.'
-        )
-    );
+            ) );
 
-    my $dialog2 = Gtk2::Label->new;
-    $scan_b->pack_start( $dialog2, FALSE, FALSE, 0 );
-    $dialog2->set_text( gettext('at this time ... ') );
+    my $label2 = Gtk2::Label->new;
+    $scan_b->pack_start( $label2, FALSE, FALSE, 0 );
+    $label2->set_text( gettext('at this time ... ') );
 
     my $time_hbox = Gtk2::HBox->new;
     $scan_b->pack_start( $time_hbox, FALSE, FALSE, 0 );
@@ -115,27 +118,36 @@ sub schedule_dialog {
     $scan_b->pack_start( $ignore_white, FALSE, FALSE, 0 );
     $ignore_white->set_active(FALSE);
 
-    my $time_hbb = Gtk2::HButtonBox->new;
-    $scan_b->pack_start( $time_hbb, FALSE, FALSE, 0 );
-    $time_hbb->set_layout('end');
+    my $time_bar = Gtk2::Toolbar->new;
+    $scan_b->pack_start( $time_bar, FALSE, FALSE, 0 );
+    $time_bar->set_style('icons');
 
-    $scan_apply_btn = Gtk2::Button->new_from_stock('gtk-add');
-    $time_hbb->add($scan_apply_btn);
+    my $dsep = Gtk2::SeparatorToolItem->new;
+    $dsep->set_draw(FALSE);
+    $dsep->set_expand(TRUE);
+    $time_bar->insert( $dsep, -1 );
+
+    $scan_apply_btn = Gtk2::ToolButton->new_from_stock('gtk-add');
+    $time_bar->insert( $scan_apply_btn, -1 );
     $scan_apply_btn->signal_connect( 'clicked' => \&apply_scan );
-    $scan_remove_btn = Gtk2::Button->new_from_stock('gtk-remove');
-    $time_hbb->add($scan_remove_btn);
+
+    $time_bar->insert( Gtk2::SeparatorToolItem->new, -1 );
+
+    $scan_remove_btn = Gtk2::ToolButton->new_from_stock('gtk-remove');
+    $time_bar->insert( $scan_remove_btn, -1 );
     $scan_remove_btn->signal_connect(
         'clicked' => sub { remove('clamtk-scan') } );
 
     my $defs_f = Gtk2::Frame->new( gettext('Antivirus signature options') );
     $vbox->pack_start( $defs_f, FALSE, FALSE, 5 );
+    #$defs_f->set_shadow_type('none');
 
     my $defs_vbox = Gtk2::VBox->new;
     $defs_f->add($defs_vbox);
 
-    my $dialog3 = Gtk2::Label->new;
-    $defs_vbox->pack_start( $dialog3, FALSE, FALSE, 5 );
-    $dialog3->set_text( gettext('Select a time to update your signatures.') );
+    my $label3 = Gtk2::Label->new;
+    $defs_vbox->pack_start( $label3, FALSE, FALSE, 5 );
+    $label3->set_text( gettext('Select a time to update your signatures.') );
 
     my $defs_hbox = Gtk2::HBox->new;
     $defs_vbox->pack_start( $defs_hbox, FALSE, FALSE, 0 );
@@ -157,16 +169,29 @@ sub schedule_dialog {
     $defs_vbox->pack_start( $defs_hbb, FALSE, FALSE, 0 );
     $defs_hbb->set_layout('end');
 
-    $defs_apply_btn = Gtk2::Button->new_from_stock('gtk-add');
-    $defs_hbb->add($defs_apply_btn);
+    my $defs_bar = Gtk2::Toolbar->new;
+    $defs_vbox->pack_start( $defs_bar, FALSE, FALSE, 0 );
+    $defs_bar->set_style('icons');
+
+    my $bsep = Gtk2::SeparatorToolItem->new;
+    $bsep->set_draw(FALSE);
+    $bsep->set_expand(TRUE);
+    $defs_bar->insert( $bsep, -1 );
+
+    $defs_apply_btn = Gtk2::ToolButton->new_from_stock('gtk-add');
+    $defs_bar->insert( $defs_apply_btn, -1 );
     $defs_apply_btn->signal_connect( 'clicked' => \&apply_defs );
-    $defs_remove_btn = Gtk2::Button->new_from_stock('gtk-remove');
-    $defs_hbb->add($defs_remove_btn);
+
+    $defs_bar->insert( Gtk2::SeparatorToolItem->new, -1 );
+
+    $defs_remove_btn = Gtk2::ToolButton->new_from_stock('gtk-remove');
+    $defs_bar->insert( $defs_remove_btn, -1 );
     $defs_remove_btn->signal_connect(
         'clicked' => sub { remove('clamtk-defs') } );
 
     my $status_f = Gtk2::Frame->new( gettext('Status') );
     $vbox->pack_start( $status_f, FALSE, FALSE, 5 );
+    #$status_f->set_shadow_type('none');
 
     my $stat_box = Gtk2::VBox->new( TRUE, 5 );
     $status_f->add($stat_box);
@@ -180,15 +205,22 @@ sub schedule_dialog {
     $defs_status->set_text(
         gettext('A daily definitions update is scheduled.') );
 
-    my $end_bbox = Gtk2::HButtonBox->new;
-    $vbox->pack_start( $end_bbox, FALSE, FALSE, 0 );
-    $end_bbox->set_layout('end');
+    my $end_bar = Gtk2::Toolbar->new;
+    $vbox->pack_start( $end_bar, FALSE, FALSE, 0 );
+    $end_bar->set_style('both-horiz');
 
-    my $close_btn = Gtk2::Button->new_from_stock('gtk-close');
-    $end_bbox->add($close_btn);
-    $close_btn->signal_connect( 'clicked' => sub { $window->destroy } );
+    my $sep = Gtk2::SeparatorToolItem->new;
+    $sep->set_draw(FALSE);
+    $sep->set_expand(TRUE);
+    $end_bar->insert( $sep, -1 );
 
-    $window->show_all();
+    my $close_btn = Gtk2::ToolButton->new_from_stock('gtk-close');
+    $close_btn->set_is_important(TRUE);
+    $end_bar->insert( $close_btn, -1 );
+    $close_btn->signal_connect( 'clicked' => sub { $dialog->destroy } );
+
+    $dialog->grab_focus();
+    $dialog->show_all();
 
     is_enabled();
     Gtk2->main();
@@ -318,14 +350,13 @@ sub apply_scan {
     my $full_cmd = $paths->{clamscan};
     $full_cmd =~ s/(.*?clamscan)\s.*/$1/;
 
+    # Home directory chosen
     if ( $home_b->get_active ) {
         $home = "-r " . $paths->{directory};
         if ( $ignore_white->get_active ) {
-
             # The user wants to scan whitelisted directories
             # $home .= ' --exclude-dir=' . $paths->{viruses};
         } else {
-
             # The box is not checked; use the whitelist
             # So add the excludes
             $home .= ' --exclude-dir=' . $paths->{viruses};
@@ -334,6 +365,7 @@ sub apply_scan {
             }
         }
     }
+    # The root (/) is chosen
     if ( $root_b->get_active ) {
         $root =
             '-r / --exclude-dir=/proc --exclude-dir=/sys --exclude-dir=/dev';
@@ -348,13 +380,26 @@ sub apply_scan {
         }
     }
 
+    # Ignore mail directories until we can parse stuff
+    for my $not_parse (
+        qw| .thunderbird	.mozilla-thunderbird
+        .evolution	Mail	kmail |
+        ) {
+        $full_cmd .= ' --exclude-dir=' . $not_parse;
+    }
+
+    # Use the appropriate signatures
     if ( ClamTk::Prefs->get_preference('Update') eq 'single' ) {
         $full_cmd .= " --database=$paths->{db}";
     }
+
+    # By default, look for PUA's
     $full_cmd .= " --detect-pua -i ";
     $full_cmd .= ( $home_b->get_active ) ? $home : $root;
-    $full_cmd
-        .= ' --log=$HOME/.clamtk/history/$(date +\%b-\%d-\%Y).log 2>/dev/null';
+
+    # Add the (ugly) logging
+    $full_cmd .= ' --log=$HOME/.clamtk/history/$(date +\%b-\%d-\%Y).log'
+        . ' 2>/dev/null';
 
     open( $T, '>>', $tmp_file ) or do {
         win_dialog('Error opening temporary file');
@@ -364,9 +409,9 @@ sub apply_scan {
     close($T);
 
     # reload crontab
-    system("$cmd $tmp_file") == 0 or do {
+    system( $cmd, $tmp_file ) == 0 or do {
         win_dialog('Error reloading cron file');
-        unlink($tmp_file);
+        unlink($tmp_file) or warn "Unable to delete tmp_file $tmp_file: $!\n";
         return;
     };
     is_enabled();
@@ -410,8 +455,7 @@ sub apply_defs {
         # It's still not a good fix since the user might not realize it...
         # But with the ability to rerun the AV choice, it should work.
         || $> != 0
-        )
-    {
+        ) {
         $full_cmd
             .= " --datadir=$paths->{db} --log=$paths->{db}/freshclam.log";
     }
@@ -435,10 +479,10 @@ sub apply_defs {
 
     # reload crontab
 
-    system("$cmd $tmp_file") == 0 or do {
+    system( $cmd, $tmp_file ) == 0 or do {
         win_dialog('Error reloading cron file');
     };
-    unlink($tmp_file);
+    unlink($tmp_file) or warn "Unable to delete tmp_file $tmp_file: $!\n";
     is_enabled();
     return;
 }
@@ -466,10 +510,10 @@ sub remove {
     close($L);
 
     # reload crontab
-    system("$cmd $tmp_file") == 0 or do {
+    system( $cmd, $tmp_file ) == 0 or do {
         win_dialog('Error reloading cron file');
     };
-    unlink($tmp_file);
+    unlink($tmp_file) or warn "Unable to delete tmp_file $tmp_file: $!\n";
 
     if ( $which eq 'clamtk-scan' ) {
         $hour_s->set_value('00');
@@ -490,14 +534,14 @@ sub win_dialog {
     my $message = "Sorry, there appears to have been an error.\n"
         . "Here is the message:\n";
     $message .= $get;
-    my $dialog;
-    $dialog =
-        Gtk2::MessageDialog->new_with_markup( $window,
+    my $popup;
+    $popup =
+        Gtk2::MessageDialog->new_with_markup( $dialog,
         [qw(modal destroy-with-parent)],
         'error', 'close', $message );
 
-    $dialog->run;
-    $dialog->destroy;
+    $popup->run;
+    $popup->destroy;
     return;
 }
 
