@@ -1,6 +1,6 @@
 # ClamTk, copyright (C) 2004-2011 Dave M
 #
-# This file is part of ClamTk.
+# This file is part of ClamTk (http://clamtk.sourceforge.net).
 #
 # ClamTk is free software; you can redistribute it and/or modify it
 # under the terms of either:
@@ -26,9 +26,6 @@ use Glib 'TRUE', 'FALSE';
 
 use Locale::gettext;
 use POSIX 'locale_h', 'strftime';
-textdomain('clamtk');
-setlocale( LC_MESSAGES, '' );
-bind_textdomain_codeset( 'clamtk', 'UTF-8' );
 
 my $required_field = "<span foreground = 'red'>*</span>";
 my $chosen = '';    # file chosen by the user for analysis
@@ -38,7 +35,7 @@ my ( $name_entry,  $email_entry,  $desc_entry );
 my ( $name_desc,   $email_desc );
 my ( $name_string, $email_string, $file_string );
 my ( $new_btn,     $close_btn,    $file_btn );
-my ( $file_status, $long_string, $bottombox, $statusbar );
+my ( $file_status, $long_string, $bottombar, $statusbar );
 
 # This file stores the number of times the
 # user has submitted a sample each day.
@@ -46,9 +43,13 @@ my $file_path = ClamTk::App->get_path('submit');
 $file_path .= '/times';
 
 sub analysis {
-    $sub_window = Gtk2::Window->new();
-    $sub_window->signal_connect( destroy => sub { $sub_window->destroy } );
-    $sub_window->set_position('center');
+    # These are the main $window's x/y coordinates.
+    # Seems to work intermittently, and possibly
+    # less expensive than sending $window itself.
+    # The ->move below shifts it to those coordinates.
+    my ( undef, $x, $y ) = @_;
+    $sub_window = Gtk2::Dialog->new();
+    $sub_window->signal_connect( close => sub { $sub_window->destroy } );
     $sub_window->set_title( change( gettext('Submit a file for analysis') ) );
     if ( -e '/usr/share/pixmaps/clamtk.png' ) {
         $sub_window->set_default_icon_from_file(
@@ -57,17 +58,18 @@ sub analysis {
         $sub_window->set_default_icon_from_file(
             '/usr/share/pixmaps/clamtk.xpm');
     }
+    $sub_window->move( $x, $y );
 
-    my $vbox = Gtk2::VBox->new( FALSE, 5 );
-    $sub_window->add($vbox);
+    my $vbox = Gtk2::VBox->new( FALSE, 1 );
+    $sub_window->get_content_area()->add($vbox);
 
     my $prelude = create_view();
-    $vbox->pack_start( $prelude, TRUE, TRUE, 5 );
+    $vbox->pack_start( $prelude, TRUE, TRUE, 1 );
 
-    my $table = Gtk2::Table->new( 3, 8, FALSE );
+    my $table = Gtk2::Table->new( 3, 7, FALSE );
     $vbox->pack_start( $table, FALSE, FALSE, 0 );
     $table->set_row_spacings(5);
-    $table->set_border_width(5);
+    $table->set_border_width(2);
 
     $name_desc = Gtk2::Label->new;
     $table->attach_defaults( $name_desc, 0, 1, 0, 1 );
@@ -86,8 +88,7 @@ sub analysis {
                 $name_entry->signal_stop_emission_by_name('insert-text');
             }
             return;
-        }
-    );
+        } );
 
     $email_desc = Gtk2::Label->new;
     $table->attach_defaults( $email_desc, 0, 1, 1, 2 );
@@ -106,8 +107,7 @@ sub analysis {
                 $email_entry->signal_stop_emission_by_name('insert-text');
             }
             return;
-        }
-    );
+        } );
 
     # The default string for the area holding the filename
     $file_string = change( gettext('No file selected') );
@@ -123,26 +123,27 @@ sub analysis {
             if ( $chosen && -e $chosen && -s $chosen ) {
                 # The following are things I've seen in filenames
                 # for malware: [a-z], [0-9], spaces, literal dots,
-                # parens, hyphens
-                if ( $chosen =~ m#^([\w\s/\.\(\)-]+)$# ) {
+                # parens, hyphens, pound (# - seen in UPS crapware),
+                # literal question marks, equals (exploit kit stuff)
+                if ( $chosen =~ m!^([#\w\s/\.\(\)\?\-=]+)$! ) {
                     $file_status->set_text( basename($1) );
                 } else {
                     $file_status->set_markup(
                         qq(<span underline="error" underline_color="red">$file_string</span>)
-                    );
+                        );
                     field_loop();
                     $file_status->set_markup($long_string);
                     $file_btn->grab_focus();
                 }
             }
-        }
-    );
+        } );
     my $f_filled_field = Gtk2::Label->new;
     $f_filled_field->set_markup($required_field);
     $table->attach_defaults( $f_filled_field, 1, 2, 2, 3 );
     $file_status = Gtk2::Label->new();
     $table->attach_defaults( $file_status, 2, 3, 2, 3 );
     $file_status->set_markup($long_string);
+    $file_status->set_ellipsize('middle');
 
     # The file can be categorized as either
     # new malware (undetected) or a false positive.
@@ -175,32 +176,42 @@ sub analysis {
                 $desc_entry->signal_stop_emission_by_name('insert-text');
             }
             return;
-        }
-    );
-
-    $table->attach_defaults( Gtk2::HSeparator->new, 0, 3, 5, 6 );
+        } );
 
     $statusbar = Gtk2::Label->new();
-    $table->attach_defaults( $statusbar, 0, 3, 6, 7 );
+    $table->attach_defaults( $statusbar, 0, 3, 5, 6 );
 
-    $bottombox = Gtk2::HButtonBox->new();
-    $table->attach_defaults( $bottombox, 0, 3, 7, 8 );
-    $bottombox->set_layout('end');
+    $bottombar = Gtk2::Toolbar->new();
+    $table->attach_defaults( $bottombar, 0, 3, 6, 7 );
+    $bottombar->set_style('both-horiz');
 
-    my $clear_btn = Gtk2::Button->new_from_stock('gtk-clear');
-    $bottombox->add($clear_btn);
+    my $ssep = Gtk2::SeparatorToolItem->new;
+    $ssep->set_draw(FALSE);
+    $ssep->set_expand(TRUE);
+    $bottombar->insert( $ssep, -1 );
+
+    my $clear_btn = Gtk2::ToolButton->new_from_stock('gtk-clear');
+    $clear_btn->set_is_important(TRUE);
+    $bottombar->insert( $clear_btn, -1 );
     $clear_btn->signal_connect( clicked => \&clear );
     $clear_btn->grab_focus();
 
-    my $cancel_btn = Gtk2::Button->new_from_stock('gtk-cancel');
-    $bottombox->add($cancel_btn);
-    $cancel_btn->signal_connect( clicked => sub { $sub_window->destroy } );
+    $bottombar->insert( Gtk2::SeparatorToolItem->new, -1 );
 
-    my $forward_btn = Gtk2::Button->new_from_stock('gtk-go-forward');
-    $bottombox->add($forward_btn);
+    my $quit_btn = Gtk2::ToolButton->new_from_stock('gtk-close');
+    $quit_btn->set_is_important(TRUE);
+    $bottombar->insert( $quit_btn, -1 );
+    $quit_btn->signal_connect( clicked => sub { $sub_window->destroy } );
+
+    $bottombar->insert( Gtk2::SeparatorToolItem->new, -1 );
+
+    my $forward_btn = Gtk2::ToolButton->new_from_stock('gtk-go-forward');
+    $forward_btn->set_is_important(TRUE);
+    $bottombar->insert( $forward_btn, -1 );
     $forward_btn->signal_connect( clicked => \&double_check );
 
-    $close_btn = Gtk2::Button->new_from_stock('gtk-close');
+    $close_btn = Gtk2::ToolButton->new_from_stock('gtk-close');
+    $close_btn->set_is_important(TRUE);
     $vbox->pack_start( $close_btn, FALSE, FALSE, 5 );
     $close_btn->signal_connect( clicked => sub { $sub_window->destroy } );
 
@@ -238,7 +249,7 @@ sub select_file {
         change( gettext('Select File') ), $sub_window, 'open',
         'gtk-cancel' => 'cancel',
         'gtk-ok'     => 'ok',
-    );
+        );
     if ( 'ok' eq $picker->run() ) {
         my $gotone = $picker->get_filename();
         $picker->destroy;
@@ -253,8 +264,7 @@ sub double_check {
     unless ( length( $name_entry->get_text )
         && length( $email_entry->get_text )
         && ( $chosen && -e $chosen )
-        && valid_email( $email_entry->get_text ) )
-    {
+        && valid_email( $email_entry->get_text ) ) {
         field_check() || return;
     }
 
@@ -280,7 +290,7 @@ sub submit {
     # Don't need buttons to click anymore.
     # If we manipulate the buttons this way, we can
     # avoid them having to be global variables.
-    my @bchild = $bottombox->get_children;
+    my @bchild = $bottombar->get_children;
     for my $l (@bchild) {
         if ( $l->isa('Gtk2::Button') ) {
             $l->set_sensitive(FALSE);
@@ -324,12 +334,11 @@ sub submit {
             name         => $description,
             sampleis     => $sampleis,
             file         => [$chosen],
-        ],
-    );
+            ],
+            );
 
     if (   $req->is_success
-        && $req->decoded_content =~ /has been successfully sent/ )
-    {
+        && $req->decoded_content =~ /has been successfully sent/ ) {
         increase_amount();
         finish('success');
     } elsif ( $req->decoded_content =~ /already recognized/ ) {
@@ -348,7 +357,7 @@ sub finish {
     my $status = shift;
     my $known  = shift;
 
-    $bottombox->hide();
+    $bottombar->hide();
 
     my $message = "\n";
     if ( $status eq 'success' ) {
@@ -356,7 +365,7 @@ sub finish {
     } elsif ( $status eq 'known' ) {
         $message =
             change( gettext('The file you submitted is already recognized') );
-        $message .= ' ';
+        $message .= "\n";
         if ($known) {
             $message .= "($known)";
         } else {
@@ -366,7 +375,7 @@ sub finish {
         $message = change(
             gettext(
                 'Unable to complete the submission. Please try again later.')
-        );
+                );
     }
     $statusbar->set_text($message);
     clear();
@@ -390,19 +399,19 @@ sub create_view {
         change( gettext('Report new viruses which are not detected') ),
         change(
             gettext('Report clean files which are incorrectly detected')
-        ),
+            ),
         change(
             gettext('Please do not submit more than two files per day.')
-        ),
-    );
+            ),
+            );
 
     my $line = "\n";
     $line .= $text[0];
     $line .= "\n\n";
-    $line .= "\t* ";
+    $line .= "* ";
     $line .= $text[1];
     $line .= "\n";
-    $line .= "\t* ";
+    $line .= "* ";
     $line .= $text[2];
     $line .= "\n\n";
     $line .= $text[3];
@@ -410,15 +419,14 @@ sub create_view {
 
     my $buffer = $view->get_buffer;
     my $iter   = $buffer->get_iter_at_offset(0);
-    $buffer->create_tag( 'deja', family     => 'DejaVu Sans' );
+    $buffer->create_tag( 'mono', family     => 'Monospace' );
     $buffer->create_tag( 'red',  foreground => 'red' );
-    $buffer->insert_with_tags_by_name( $iter, $line, 'deja' );
+    $buffer->insert_with_tags_by_name( $iter, $line, 'mono' );
     $buffer->insert_with_tags_by_name( $iter, '* ',  'red' );
     $buffer->insert_with_tags_by_name( $iter,
-        change( gettext('Indicates a required field') ), 'deja' );
+        change( gettext('Indicates a required field') ), 'mono' );
 
     $sw->add($view);
-
     return $sw;
 }
 
@@ -439,7 +447,7 @@ sub field_check {
     if ( !length( $name_entry->get_text() ) || !length($tmp) ) {
         $name_desc->set_markup(
             qq(<span underline="error" underline_color="red">$name_string</span>)
-        );
+            );
         field_loop();
         $name_desc->set_markup($name_string);
         $name_entry->grab_focus();
@@ -453,7 +461,7 @@ sub field_check {
     if ( !length( $email_entry->get_text() ) || !length($tmp) ) {
         $email_desc->set_markup(
             qq(<span underline="error" underline_color="red">$email_string</span>)
-        );
+            );
         field_loop();
         $email_desc->set_markup($email_string);
         $email_entry->grab_focus();
@@ -464,7 +472,7 @@ sub field_check {
     if ( !-e $chosen ) {
         $file_status->set_markup(
             qq(<span underline="error" underline_color="red">$file_string</span>)
-        );
+            );
         field_loop();
         $file_status->set_markup($long_string);
         $file_btn->grab_focus();
@@ -489,8 +497,7 @@ sub field_loop {
         sub {
             $loop->quit;
             FALSE;
-        }
-    );
+        } );
     $loop->run;
     return;
 }
@@ -656,7 +663,7 @@ xff]|\((?:[^\\\x80-\xff\n\015()]|\\[^\x80-\xff])*\))*\))*\.(?:[\040\t]|\((?
     } else {
         $email_desc->set_markup(
             qq(<span underline="error" underline_color="red">$email_string</span>)
-        );
+            );
         field_loop();
         $email_desc->set_markup($email_string);
         $email_entry->grab_focus();
