@@ -1,6 +1,6 @@
 # ClamTk, copyright (C) 2004-2011 Dave M
 #
-# This file is part of ClamTk.
+# This file is part of ClamTk (http://clamtk.sourceforge.net).
 #
 # ClamTk is free software; you can redistribute it and/or modify it
 # under the terms of either:
@@ -21,9 +21,6 @@ use encoding 'utf8';
 
 use Locale::gettext;
 use POSIX 'locale_h';
-textdomain('clamtk');
-setlocale( LC_MESSAGES, '' );
-bind_textdomain_codeset( 'clamtk', 'UTF-8' );
 
 use Gtk2::SimpleList;
 use Glib 'TRUE', 'FALSE';
@@ -33,10 +30,16 @@ my ( $go_btn, $close_btn, $cancel_btn );
 my ($update_sig_pid);
 
 sub update_dialog {
+    # These are the main $window's x/y coordinates.
+    # Seems to work intermittently, and possibly
+    # less expensive than sending $window itself.
+    # The ->move below shifts it to those coordinates.
+    my ( undef, $x, $y ) = @_;
     $win = Gtk2::Dialog->new();
     $win->signal_connect( destroy => sub { $win->destroy; } );
-    $win->set_title( gettext('Updates') );
+    $win->set_title( gettext('Virus Scanner') );
     $win->set_default_size( 625, 130 );
+    $win->move( $x, $y );
 
     if ( -e '/usr/share/pixmaps/clamtk.png' ) {
         $win->set_default_icon_from_file('/usr/share/pixmaps/clamtk.png');
@@ -56,8 +59,9 @@ sub update_dialog {
         gettext('Description') => 'text',
         gettext('Select')      => 'bool',
         gettext('Status')      => 'text',
-    );
+        );
     $update_win->add($update_list);
+    $update_win->grab_focus();
 
     my $sig_box = Gtk2::CheckButton->new();
     $sig_box->set_active(FALSE);
@@ -106,34 +110,43 @@ sub update_dialog {
 
     # This box will hold buttons to check for updates, cancel,
     # close the window, and if the user's root, warn them.
-    my $hbox = Gtk2::HButtonBox->new();
-    $win->vbox->pack_start( $hbox, FALSE, FALSE, 0 );
-    $hbox->set_layout_default('spread');
-    $win->vbox->set_focus_child($hbox);
+    my $choicebar = Gtk2::Toolbar->new();
+    $choicebar->set_style('both-horiz');
+    $win->vbox->pack_start( $choicebar, FALSE, FALSE, 0 );
+    $win->vbox->set_focus_child($choicebar);
+
+    my $u_sep = Gtk2::SeparatorToolItem->new;
+    $u_sep->set_draw(FALSE);
+    $u_sep->set_expand(TRUE);
+    $choicebar->insert( $u_sep, -1 );
 
     # Check for updates button
-    my $go_img = Gtk2::Image->new_from_stock( 'gtk-redo', 'small-toolbar' );
-    $go_btn = Gtk2::Button->new();
-    $go_btn->signal_connect( clicked => \&decision );
-    $go_btn->set_property( 'image' => $go_img );
+    $go_btn = Gtk2::ToolButton->new_from_stock('gtk-network');
+    $go_btn->set_is_important(TRUE);
+    $go_btn->set_expand(FALSE);
     $go_btn->set_label( gettext('Check for updates') );
-    $tt->set_tip( $go_btn, gettext('Check for updates') );
-    $hbox->add($go_btn);
+    $go_btn->signal_connect( clicked => \&decision );
+    $choicebar->insert( $go_btn, -1 );
+
+    $choicebar->insert( Gtk2::SeparatorToolItem->new, -1 );
 
     # Close window button
-    $close_btn = Gtk2::Button->new_from_stock('gtk-close');
-    $hbox->add($close_btn);
+    $close_btn = Gtk2::ToolButton->new_from_stock('gtk-close');
+    $close_btn->set_is_important(TRUE);
+    $close_btn->set_expand(FALSE);
     $close_btn->signal_connect(
         clicked => sub {
             $update_win->destroy;
             $win->destroy;
-        }
-    );
-    $tt->set_tip( $close_btn, gettext('Close this window') );
+        } );
+    $choicebar->insert( $close_btn, -1 );
+
+    $choicebar->insert( Gtk2::SeparatorToolItem->new, -1 );
 
     # Cancel button
-    $cancel_btn = Gtk2::Button->new_from_stock('gtk-cancel');
-    $hbox->add($cancel_btn);
+    $cancel_btn = Gtk2::ToolButton->new_from_stock('gtk-cancel');
+    $cancel_btn->set_is_important(TRUE);
+    $cancel_btn->set_expand(FALSE);
     $cancel_btn->signal_connect(
         clicked => sub {
             kill 15, $update_sig_pid if ($update_sig_pid);
@@ -144,14 +157,15 @@ sub update_dialog {
             $go_btn->set_sensitive(TRUE);
             $close_btn->set_sensitive(TRUE);
             $cancel_btn->set_sensitive(FALSE);
-        }
-    );
+        } );
+    $choicebar->insert( $cancel_btn, -1 );
     $cancel_btn->set_sensitive(FALSE);
 
     # Warn the user not to do things as root.
     if ( $> == 0 ) {
-        my $warning_btn = Gtk2::Button->new_from_stock('gtk-dialog-warning');
-        $hbox->add($warning_btn);
+        my $warning_btn =
+            Gtk2::ToolButton->new_from_stock('gtk-dialog-warning');
+        $choicebar->insert( $warning_btn, -1 );
         $warning_btn->signal_connect(
             clicked => sub {
                 my $message = gettext(
@@ -164,8 +178,7 @@ sub update_dialog {
 
                 $dialog->run;
                 $dialog->destroy;
-            }
-        );
+            } );
     }
 
     # Rotate (actually truncate) user's freshclam.log
@@ -372,7 +385,7 @@ sub update_gui {
     if ( $caller and $caller eq 'startup' ) {
         # The user may have set the preference ('GUICheck')
         # to not check this on startup
-        return -1
+        return 5
             unless ( ClamTk::Prefs->get_preference('GUICheck') );
     }
 
@@ -399,7 +412,7 @@ sub update_gui {
                             $url = $1;
                         }
                         last if ( !$url );
-                        if (/HTTPProxyPort\s+(.*?)$/) {
+                        if (/HTTPProxyPort\s+(\d+)$/) {
                             $port = $1;
                         }
                     }
@@ -430,8 +443,15 @@ sub av_db_select {
     # This is an easy way for the user to switch back
     # and forth their preferred way to update signatures.
 
+    # These are the main $window's x/y coordinates.
+    # Seems to work intermittently, and possibly
+    # less expensive than sending $window itself.
+    # The ->move below shifts it to those coordinates.
+    my ( undef, $x, $y ) = @_;
+
     my $assistant = Gtk2::Assistant->new;
-    $assistant->set_default_size( -1, 240 );
+    #$assistant->set_default_size( -1, 240 );
+    $assistant->move( $x, $y );
 
     # What the user already has - and then chooses.
     # We'll declare it now because of the button toggle action.
@@ -463,7 +483,7 @@ sub av_db_select {
     # This ButtonBox will hold the manual/automatic option.
     my $choice_bb = Gtk2::HButtonBox->new;
     $abox->pack_start( $choice_bb, FALSE, FALSE, 0 );
-    # The 'center' layout might be nicer, but CentOS 5.5 has
+    # The 'center' layout might be nicer, but CentOS 5.x has
     # an older version of Gtk2 apparently, so we'll use 'spread'.
     $choice_bb->set_layout('spread');
 
@@ -474,8 +494,7 @@ sub av_db_select {
     $manual_btn->signal_connect(
         toggled => sub {
             $pref = 'single';
-        }
-    );
+        } );
 
     # Automatically update the sigs.
     # This is the default, out-of-the-box choice.
@@ -486,8 +505,7 @@ sub av_db_select {
     $auto_btn->signal_connect(
         toggled => sub {
             $pref = 'shared';
-        }
-    );
+        } );
 
     # We'll start with what the user currently has selected
     # as their update preference.
@@ -515,22 +533,22 @@ sub av_db_select {
     # There's an option already there, so set it complete no matter what.
     $assistant->set_page_complete( $abox, TRUE );
 
-    # The Assistant's callbacks
+    # The Assistant's signals/callbacks
     $assistant->signal_connect(
         cancel => sub {
             $assistant->destroy;
-        }
-    );
+        } );
     $assistant->signal_connect(
         close => sub {
             $assistant->destroy;
-        }
-    );
+        } );
     $assistant->signal_connect(
         apply => sub {
-            save($pref);
-        }
-    );
+            save( $auto_btn->get_active ? 'shared' : 'single' );
+            # Update status of signatures.  The date and status
+            # (Current/Outdated) may have changed when switching.
+            ClamTk::GUI->set_sig_status();
+        } );
 
     # This is the second page: 'confirm' or confirmation.
     my $comp_page = Gtk2::Alignment->new( 0.5, 0.5, 0.5, 0.0 );
@@ -561,18 +579,16 @@ sub save {
     if ( $ret == 1 ) {
         # It worked, so see if there are system signatures around
         # we can copy to save bandwidth and time
-
         my $paths = ClamTk::App->get_path('db');
 
         if ( $update eq 'single' ) {
             my ( $d, $m ) = (0) x 2;
             Gtk2->main_iteration while ( Gtk2->events_pending );
             for my $dir_list (
-                '/var/lib/clamav',         '/var/clamav',
+                '/var/clamav',             '/var/lib/clamav',
                 '/opt/local/share/clamav', '/usr/share/clamav',
                 '/usr/local/share/clamav', '/var/db/clamav',
-                )
-            {
+                ) {
                 if ( -e "$dir_list/daily.cld" ) {
                     system( 'cp', "$dir_list/daily.cld", "$paths/daily.cld" );
                     $d = 1;
